function [phi,alpha,uvdm,err] = vdm_approx(u,nfft)
%VDM_APPROX Vandermonde vector approximation
% [phi,alpha,uvdm,err] = vdm_approx(u,method)
%
% Given a vector u, approximate it by a a Vandermonde vector uvdm such that  
%               u = uvdm + n, 
% where: 
% - n is the length-P vector of residuals
% - uvdm = alpha*h
% - alpha a scaling factor 
% - h=[1, exp(j*phi), exp(2*j*phi), ..., exp((P-1)*j*phi)].'
%
% Method used: estimation of phi from the location of the peak of the periodogram [1]
%
% INPUTS:
% - u : vector to be approximated by uvdm
% - nfft (default 2^16): number of points in the periodogram
%
% OUTPUTS:
% - phi :   estimate of the angle
% - alpha : estimate of the scaling factor
% - uvdm = alpha*h  : vdm vector approximate
% - err = norm(u-uvdm,'fro') : norm of residual

% @Copyright 2008
% Dimitri Nion (feedback: dimitri.nion@gmail.com)
%
% For non-commercial use only

% [1] Rife, D.C., Boorstyn, R.R.: Single-tone parameter estimation from 
% discrete-time observations. IEEE Trans. Inform. Theory IT 20(5), 591–598 (1974)

u=u(:);   % to get a column-vector
P=length(u);
if exist('nfft')==0 || isempty(nfft);
   nfft=2^16;   % useful for zero-padding the fft
end
sp=abs(fft(u/norm(u),nfft));
[maxval, del]=max(sp);   % localize the peak of the fft
if del>nfft/2+1
   del=nfft-del;
else
   del=-del;
end
del=del+1;
phi=-2*del*pi/nfft;
alpha=(1/P)*exp(-j*phi*(0:P-1))*u;
uvdm=alpha*exp(j*phi*(0:P-1)).';
err=norm(u-uvdm,'fro');
